# Datasets
VARIANT_SCORE_DATA = "score_data"
VARIANT_COUNT_DATA = "count_data"

# Dataset columns
HGVS_NT_COLUMN = "hgvs_nt"
HGVS_SPLICE_COLUMN = "hgvs_splice"
HGVS_PRO_COLUMN = "hgvs_pro"
HGVS_COLUMNS = sorted([HGVS_NT_COLUMN, HGVS_PRO_COLUMN, HGVS_SPLICE_COLUMN])

# META_DATA = 'meta_data'

SCORE_COLUMNS = "score_columns"
COUNT_COLUMNS = "count_columns"

REQUIRED_SCORE_COLUMN = "score"
